/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"



int 
main(int  argc, 
     char **argv)
{
  bool          done        = false;
  bool          gray        = false;
  int           adapt       = 1;
  tlDisplay     *window     = new tlDisplay ();
  tlImage       *image      = new tlImage   ();
  tlBackground  *background = new tlBackground ();
  TL_DIGITIZER  *digitizer  = new TL_DIGITIZER ();


  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // initialize objects
  window->show ();
  if (digitizer->open(0) != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

  // init background, learning mode
  digitizer->grab (image);
  if (gray) image->toGray ();
  background->init (image, 1, 1);

  // main loop
  while (!done) 
  {
    // grab image
    digitizer->grab (image);

    // get background
    if (gray) image->toGray ();
    background->update (image);
    background->filter (image);

    // display image
    window->display (image);

    // event management
    if (tlDisplay::queryEvent ()) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_EXIT:
        done = true;
        break;
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case 'g':
          gray = !gray;
        case 'r':
          digitizer->grab (image);
          if (gray) image->toGray ();
          background->init (image, 1, adapt);
          break;
        case 'a':
          if (adapt) adapt = 0; else adapt = 1;
          background->setAdapt (adapt);
          break;
        case 'q':
        case 'Q':
          done = true;
          break;
        }
      }
    }
  }


  // cleanup
  delete image;
  delete background;
  delete digitizer;
  delete window;
  printf("done.\n");


  return 0;
}

