/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <stdlib.h>

#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"

#ifdef WIN32
#include <windows.h>
#endif



int 
main (int  argc, 
      char **argv)
{
  int done    = false;
  int capture = false;
  int index   = 0;
  int fps     = 10;


  // if command line argument, set fps
  if (argc == 2) {
    fps = atoi (argv[1]);
  }

  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (0);

  // initialize TLIB vars
  tlDisplay    *window    = new tlDisplay ("Color capture");
  TL_DIGITIZER *digitizer = new TL_DIGITIZER();
  tlImage      *image     = new tlImage ();

  // open digitizer
  if (digitizer->open () != TL_NO_ERROR) {
    TL_ERROR_MSG ("error - digitizer failed to open");
    return TL_ERROR;
  }

  // set framerate
  if (digitizer->setFrameRate (fps) != TL_NO_ERROR) {
    TL_ERROR_MSG ("warning - digitizer failed to set frame rate");
  }

  // show display window
  window->show ();

  // show exit message
  printf ("press 's' to start/stop capture\n");
  printf ("press 'q' to quit\n");

  // main loop
  while (!done) 
  {
    // grab image
    if ((digitizer->grab (image)) != TL_NO_ERROR) {
      TL_ERROR_MSG ("error - grab failed");
      break;
    }

    // display image
    if ((window->display (image)) != TL_NO_ERROR) {
      if (!done) TL_ERROR_MSG ("error - display failed");
      break;
    }

    // write image to disk
    if (capture) {
      char filename[255];
      sprintf (filename, "image%d.tlb", index++);
      printf ("%.2f writing %s\n", tl_timer_get_time() / 1000.0f, filename);
      image->writeToFile (filename);
    }

    // event management
    if (tlDisplay::queryEvent () != TL_NO_EVENT) {
      if (tlDisplay::event() == TL_EVENT_KEYDOWN) {
        switch (tlDisplay::getKeyDown ()) {  
        case 's':
        case 'S':
          capture = !capture;
          if (capture) {
            tl_timer_init();
          }
          break;
        case 'q':
        case 'Q':
          done = true;
          break;
        }
      }
    }
  }

  // cleanup
  delete image;
  delete digitizer;
  delete window;


  return (0);
}
