/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <string.h>

#include "tlVision.h"
#include "tlDisplay.h"

#ifdef WIN32
#include <windows.h>
#endif



enum conversion {PPM, JPG, BMP};



int 
main (int  argc, 
      char **argv)
{
  int     conversion  = PPM;
  char    *filename;
  tlImage *image      = new tlImage ();


  // usage
  if (argc < 2) {
    printf ("usage: %s [bmp] [jpg] file.tlb [...]\n", argv[0]);
    printf ("       bmp   convert to BMP instead of PPM\n");
    printf ("       jpg   convert to JPEG instead of PPM\n");
    return (0);
  }

  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (0);

  // process args
  for (int i=1; i<argc; i++) {
    if (!strcmp (argv[i], "bmp")) {
      conversion = BMP;
      continue;
    }
    else if (!strcmp (argv[i], "jpg")) {
      conversion = JPG;
      continue;
    }
    filename = argv[i];

    // read in image
    if (TL_NO_ERROR == image->readFromFile (filename)) {
      char outname[256];
      switch (conversion) {
      case BMP:
        sprintf (outname, "%s.bmp", filename);
        image->writeToBMP (outname);
        break;
      case PPM:
        sprintf (outname, "%s.ppm", filename);
        image->writeToPPM (outname);
        break;
      case JPG:
        sprintf (outname, "%s.jpg", filename);
        image->writeToJPG (outname);
        break;
      }
      printf ("converted %s -> %s\n", filename, outname);
    }
    else {
      printf ("error: unable to read %s\n", filename);
    }
  }

  // cleanup
  delete image;


  return (0);
}
