/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"


#define KERNEL_SIZE 3


int 
main(int  argc, 
     char **argv)
{
  bool          done        = false;
  float         kernel[KERNEL_SIZE*KERNEL_SIZE];
  tlDisplay     *window     = new tlDisplay ();
  tlImage       *image      = new tlImage   ();
  TL_DIGITIZER  *digitizer  = new TL_DIGITIZER ();


  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // initialize objects
  window->show ();
  if (digitizer->open(0) != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

  // initialize convolution kernel
  for (int i=0; i<KERNEL_SIZE*KERNEL_SIZE; i++) {
    kernel[i] = 1.0f/(float)(KERNEL_SIZE*KERNEL_SIZE);
  }

  // main loop
  while (!done) 
  {
    // grab imag
    if ((digitizer->grab (image)) != TL_NO_ERROR) {
      TL_ERROR_MSG ("error - grab failed (digitizer 1)");
      break;
    }

    image->convolution (kernel, KERNEL_SIZE);

    // display image
    if ((window->display (image)) != TL_NO_ERROR) {
      if (!done) TL_ERROR_MSG ("error - display failed");
      break;
    }
    
    // event management
    if (tlDisplay::queryEvent ()) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_EXIT:
        done = true;
        break;
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case 'q':
        case 'Q':
          done = true;
          break;
        }
      }
    }
  }


  // cleanup
  delete image;
  delete digitizer;
  delete window;
  printf("done.\n");


  return 0;
}
