/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <string.h>

#include "tlVision.h"
#include "tlDisplay.h"

#ifdef WIN32
#include <windows.h>
#endif



int 
main (int  argc, 
      char **argv)
{
  char  *filename;
  int   done        = false;
  int   event       = TL_NO_EVENT;
  int   automatic   = false;


  // usage
  if (argc < 2) {
    printf ("usage: %s [-a] file.tlb [...]\n", argv[0]);
    printf ("       -a   automatically proceed to next image at 10 Hz\n");
    return (0);
  }

  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (0);

  // initialize TLIB vars
  tlDisplay   *window    = new tlDisplay ("Color capture");
  tlImage     *image     = new tlImage ();
  window->show();

  // show exit message
  printf ("press 'Q' at anytime to quit\n");

  // process
  for (int i=1; (i<argc) && !done; i++) {
    if (!strcmp (argv[i], "-a")) {
      automatic = true;
      i++;
    }

    filename = argv[i];

    printf ("\nreading: %s\n", filename);
    image->readFromFile (filename);

    printf ("click image to proceed\n");
    window->display (image);

    do {
      if (tlDisplay::queryEvent() == TL_EVENT) {
        event = tlDisplay::event();
      }
      if (event == TL_EVENT_KEYDOWN) {
        if (tlDisplay::getKeyDown() == 'Q') {
          done = true;
        }
      }
    } while (!done && !automatic && (event != TL_EVENT_LMOUSEDOWN));
    event = TL_NO_EVENT;
    
    if (automatic) {
      tlThreadSleep (100);
    }
  }

  // cleanup
  delete image;
  delete window;


  return (0);
}
