/******************************************************************************
 *
 * (c) Copyright 2000-2003 T. Fong, S. Grange
 *     EPFL - Swiss Federal Institute of Technology
 *     All rights reserved.
 *  
 * FILE:     DisplayLive.cpp
 *
 * ABSTRACT: Simple image display using file digitizer (tlSourceFiles)
 *
 * REVISION: $Id: DisplayLive.cpp,v 1.1 2004-12-01 15:21:04 seb Exp $
 *
 *****************************************************************************/

#include <stdlib.h>		// exit

#include "tlSourceFiles.h"
#include "tlDisplay.h"
#include "tlTimer.h"
#include "tlVision.h"

// globals
tlDisplay     *display    = NULL;
tlSourceFiles *digitizer;


////////////////////////////////////////////////////////////////////////
//
//  Initialization
//
////////////////////////////////////////////////////////////////////////
int init ()
{
  // library init
  tl_init_lib ();

  // verbosity mode
  tl_set_verbose (0);

  // open window
  display = new tlDisplay ("SimpleApp");

  // open digitizer
  digitizer = new tlSourceFiles ();

  if (digitizer->open() != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

  if (digitizer->setFrameRate (20) != TL_NO_ERROR) {
    TL_ERROR_MSG ("error - digitizer failed to set frame rate");
    return TL_ERROR;
  }

  // show windows
  display->show ();
  
  return TL_NO_ERROR;
}


////////////////////////////////////////////////////////////////////////
//
//  Clean up.
//
////////////////////////////////////////////////////////////////////////
int cleanup ()
{
  delete digitizer;
  delete display;

  return TL_NO_ERROR;
}


////////////////////////////////////////////////////////////////////////
//
//  Application entry point
//
////////////////////////////////////////////////////////////////////////
int
main (int  argc, char **argv)
{
  int done = 0, frame = 0;
  tlImage *image = new tlImage ();

  // initialization
  if (init () != TL_NO_ERROR) {
    exit (1);
  }

  // 
  // main loop
  // 

  while (!done) {
    // image processing
    digitizer->grab (image);
    display->display (image);

    // event processing
    if (tlDisplay::queryEvent() == TL_EVENT) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_KEYDOWN:
	printf ("key down---> %c\n", tlDisplay::getKeyDown());
	if (tlDisplay::getKeyDown() == 'q') {
	  printf ("quitting\n");
	  done = 1;
	}
	break;

      case TL_EVENT_KEYUP:
	printf ("key up---> %c\n", tlDisplay::getKeyUp());
	break;

      case TL_EVENT_MOUSEDOWN:
	printf ("mouse down\n");
	break;

      case TL_EVENT_MOUSEUP:
	printf ("mouse up\n");
	break;

      case TL_EVENT_MOUSEDBLCLK:
	printf ("mouse double click\n");
	break;

      case TL_EVENT_EXIT:
        done = 1;
	break;
      }
    }


    //
    // statistics 
    //
    if (++frame == 1) {
      if (tl_timer_init() != TL_NO_ERROR) {
	exit (1);
      }
    }
    if (!(frame % 30)) {
      printf ("%.1f frames/sec\n",
	      (float) frame / ((float) tl_timer_get_time() / 1000.0));
    }

  }


  // cleanly exit
  delete image;

  cleanup ();
  printf("done.\n");

  return 0;
}
