/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <stdlib.h>

#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlTimer.h"
#include "tlVision.h"

// uncomment the following to read image from camera instead of "src.jpg"
// #define USE_CAMERA

// globals
tlDisplay     *display      = NULL;
#ifdef USE_CAMERA
TL_DIGITIZER  *digitizer;
#endif



// Initialization routine

int 
init (int  argc, 
      char **argv)
{
  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // verbosity mode
  tl_set_verbose (0);

  // open window
  display = new tlDisplay ("SimpleApp");

  // show windows
  display->show ();

#ifdef USE_CAMERA
  // open digitizer
  digitizer = new TL_DIGITIZER();

  if (digitizer->open () != TL_NO_ERROR) {
    TL_ERROR_MSG ("error - digitizer failed to open");
    return TL_ERROR;
  }
#endif


  return TL_NO_ERROR;
}



// Clean up and exit 

void
cleanup (int exitcode)
{
  delete display;
#ifdef USE_CAMERA
  delete digitizer;
#endif

  exit (exitcode);
}



// Application entry point
int
main (int  argc, 
      char **argv)
{
  long      dt;
  tlImage   *imageSrc   = new tlImage ();
  tlImage   *imageEdge  = new tlImage ();
  

  // initialization
  if (init (argc, argv) != TL_NO_ERROR) {
    exit (1);
  }
  
#ifdef USE_CAMERA
  // display exit message
  printf ("\nclick the image to capture\n");
  
  // event processing - detect mouse clicks
  while (1) {
    if (tlDisplay::queryEvent() == TL_EVENT) {
      if (tlDisplay::event() == TL_EVENT_LMOUSEDOWN) {
        break;
      }
    }
    if (TL_NO_ERROR != digitizer->grab (imageSrc)) {
      printf ("image capture failed\n");
      cleanup (1);
    }
 
    // display image
    display->display (imageSrc);
  }
  
#else
  if (TL_NO_ERROR != imageSrc->readFromJPG ("src.jpg")) {
    printf ("image read failed\n");
    cleanup (1);
  }
#endif

  // original
  printf ("\noriginal image (%dx%d)\n", imageSrc->width, imageSrc->height);
  display->display (imageSrc);
  printf ("click the image to continue\n");
  tlDisplay::waitMouse();

  // morphological edge filter
  tl_timer_init();
  imageSrc->edges (imageEdge, TL_EDGES_MORPH);
  dt = tl_timer_get_time();
  printf ("\nedge detect with morphological filters (%ld msec)\n", dt);
  display->display (imageEdge);
  printf ("click the image to continue\n");
  tlDisplay::waitMouse();

  // sobel edge filter
  tl_timer_init();
  imageSrc->edges (imageEdge, TL_EDGES_SOBEL);
  dt = tl_timer_get_time();
  printf ("\nedge detect with Sobel detector (%ld msec)\n", dt);
  display->display (imageEdge);
  printf ("click the image to continue\n");
  tlDisplay::waitMouse();

  // prewitt edge filter
  tl_timer_init();
  imageSrc->edges (imageEdge, TL_EDGES_PREWITT);
  dt = tl_timer_get_time();
  printf ("\nedge detect with Prewitt detector (%ld msec)\n", dt);
  display->display (imageEdge);
  printf ("click the image to continue\n");
  tlDisplay::waitMouse();

  delete imageSrc;
  delete imageEdge;

  cleanup (0);


  return 0;
}
