/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"



int 
main(int  argc, 
     char **argv)
{
  bool           done        = false;
  tlDisplay      *window     = new tlDisplay   ();
  TL_DIGITIZER   *digitizer  = new TL_DIGITIZER();
  tlImage        *image      = new tlImage     ();
  tlImage        *hImage     = new tlImage     ();
  char           tmpstr[255];


  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);
  
  // initialize objects
  window->show ();
  if (digitizer->open() != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }
  
  // why not do it fast ?
  digitizer->setFrameRate (60);

  // initialize hough engine
  tlHough *hough = new tlHough (digitizer->getWidth(), digitizer->getHeight());

  // main loop
  while (!done) 
  {
    // generate a quick and dirty thin-edge image
    digitizer->grab (image);
    image->toGray (hImage);
    hImage->edges (hImage, TL_EDGES_SOBEL);
    hImage->normalize();
    hImage->toBin (200);
    hImage->thin ();

    // extract the single most promising line
    hough->extract (hImage, 1);

    // draw the line and display its angle on the source image
    if (hough->getLineCount() > 0) {
      tlLine *line = hough->getLinePtr (0);
      image->drawLine (line, TL_COLOR_GREEN);
      sprintf (tmpstr, "%.1f deg", (line->theta*180.0/TL_PI));
      int tx = TL_MIN(line->p1->x+5, 280);
      int ty = TL_MIN(line->p1->y+5, 220);
      image->drawText (tmpstr, tx, ty, TL_COLOR_GREEN);
    }
    
    // display image
    if ((window->display (image)) != TL_NO_ERROR) {
      if (!done) TL_ERROR_MSG ("error - display failed");
      break;
    }

    // event management
    if (tlDisplay::queryEvent ()) {
      switch (tlDisplay::event ()) {
      case TL_EVENT_EXIT:
        done = true;
        break;
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case 'q':
        case 'Q':
          done = true;
          break;
        }
      }
    }
  }


  // cleanup
  delete image;
  delete window;
  printf("done.\n");


  return 0;
}
