/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include <stdio.h>
#include <stdlib.h>

#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"

#ifdef WIN32
#include <windows.h>
#endif


tlDisplay     *display    = NULL;
TL_DIGITIZER  *digitizer;

#define DETECTION_REDEFINE 95
#define DETECTION_LOST     50



// Initialization routine

int 
init (int  argc,
      char **argv)
{
  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // verbosity mode
  tl_set_verbose (1);

  // open windows
  display = new tlDisplay ("live");

  // open digitizer
  digitizer = new TL_DIGITIZER();
  if (digitizer->open() != TL_NO_ERROR) {
    TL_ERROR_MSG ("error opening digitizer\n");
    return TL_ERROR;
  }

  // the faster the better !
  digitizer->setFrameRate (60);

  // show windows
  display->show ();

  
  return TL_NO_ERROR;
}



//  Clean up routine

int 
cleanup (void)
{
  delete digitizer;
  delete display;


  return TL_NO_ERROR;
}



// Application entry point

int
main (int  argc,
      char **argv)
{
  // vars
  int done   = 0;
  int res;
  int status = TL_ERROR;


  // initialization
  if (init (argc, argv) != TL_NO_ERROR) exit(1);

  // TLIB vars
  tlImage  *image     = new tlImage  ();
  tlImage  *refImage  = new tlImage  ();
  tlObject *object    = new tlObject ();
  tlRect   *rect      = new tlRect   ();
  tlPoint  *point;

  // main loop
  while (!done) { 
    // image processing
    digitizer->grab (image);

    // invert comments to use another color-mode
    image->toGray (refImage);
    // image->toNrg (refImage);   // NRG
    // image->copyTo (refImage);  // RGB
    
    // event processing
    if (tlDisplay::queryEvent ()) {
      switch (tlDisplay::event ()) {    
      case TL_EVENT_EXIT:
        done = 1;
        break; 
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case 'Q': 
        case 'q': 
          done = 1;
          break;
        }
        break;
      case TL_EVENT_MOUSERECT:
        tlDisplay::getMouseRect (rect);
        status = object->define (refImage, rect);
        object->toRect (rect);
        break;
      } 
    }

    // tracking
    if (status == TL_NO_ERROR || status == TL_ERROR_STACK_OVERFLOW) {
      
      // predict object location
      point = object->getPredictedPosition ();
      object->toRect (rect);
      rect->setLocation (point);
      rect->expand (20);

      // find object
      res = refImage->findBest (object, rect);

      // excellent match, redefine
      if (res > DETECTION_REDEFINE) {
        object->toRect (rect);
        status = object->define (refImage, rect);
      }

      // bad match, try larger window in case object moved outside searching box
      else if (res < DETECTION_LOST) {
        rect->expand (20);
        res = refImage->findBest (object, rect);

        // if no match, we definitely lost the object
        if (res < DETECTION_LOST) {
          printf ("object lost (match is %d %%)   \n", res);
          status = TL_ERROR_NOT_FOUND;
        }
      }
      
      printf ("template matching %d %% \r", res);

      // display information on image
      if (res > DETECTION_REDEFINE) image->drawCross (object, TL_COLOR_GREEN);
      else {
        image->drawCross (object, TL_COLOR_RED);
        image->drawRect (rect, TL_COLOR_WHITE);
      }
    }
    
    display->display (image);
  }


  // cleanly exit
  delete image;
  delete refImage;
  delete object;
  delete rect;
  cleanup ();
  printf("done.\n");


  return 0;
}

