/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlCameraSVSHead.h"
#include "tlDisplay.h"
#include "tlVision.h"



int 
main(int  argc, 
     char **argv)
{
  bool            done        = false;
  char            tmpstr[8];
  tlCameraSVSHead *camera     = new tlCameraSVSHead ();
  tlPoint         *point      = new tlPoint         ();
  tlDisplay       *window     = new tlDisplay       ("source image",  100, 100);
  tlDisplay       *dispwin    = new tlDisplay       ("disparity map", 150+camera->getWidth(), 100);
  tlImage         *image      = new tlImage         ();
  tlImage         *disparity  = new tlImage         ();


  // initialize TLIB
  tl_init_lib (argc, argv);
  tl_set_verbose (1);

  // initialize objects
  window->show ();
  dispwin->show ();

  // main loop
  while (!done) 
  {
    // grab image
    if ((camera->grab (image, disparity)) != TL_NO_ERROR) {
      TL_ERROR_MSG ("error - grab failed");
      break;
    }

    // compute display info
    point->disp = disparity->getPixel (point);
    image->drawCross (point, 7, TL_COLOR_BLACK);
    if (point->disp > -1) {
      camera->toWorld (point);
      sprintf (tmpstr, "%.2f", point->wz);
      image->drawText (tmpstr, point->x+4, point->y+4, TL_COLOR_BLACK);
    }

    // we must convert disparity to a gray image so we can draw things on it
    disparity->toGray();
    disparity->drawCross (point, 7, TL_COLOR_WHITE);
    sprintf (tmpstr, "%d", point->disp);
    disparity->drawText (tmpstr, point->x+4, point->y+4, TL_COLOR_WHITE);

    // display images
    if ((window->display (image)) != TL_NO_ERROR) {
      if (!done) TL_ERROR_MSG ("error - display failed");
      break;
    }
    if ((dispwin->display (disparity)) != TL_NO_ERROR) {
      if (!done) TL_ERROR_MSG ("error - display failed");
      break;
    }


    // event management
    if (tlDisplay::queryEvent () != TL_NO_EVENT) {
      if (tlDisplay::event () == TL_EVENT_EXIT) done = true;
      switch (tlDisplay::getLastEvent ()) {
      case TL_EVENT_LMOUSEUP:
        tlDisplay::getMouseUp (point);
        break;
      case TL_EVENT_KEYUP:
        switch (tlDisplay::getKeyUp ()) {
        case '.':
          camera->setOffX (camera->getOffX()+1);
          printf ("offx = %d\n", camera->getOffX());
          break;
        case ',':
          camera->setOffX (camera->getOffX()-1);
          printf ("offx = %d\n", camera->getOffX());
          break;
        case 'c':
          if (camera->isColor()) camera->enableColor (false);
          else                   camera->enableColor (true);
          printf ("color toggle\n");
          break;
        case 'p':
          if (camera->isSTOC()) camera->enableSTOC (false);
          else                  camera->enableSTOC (true);
          printf ("STOC toggle\n");
          break;
        case 'q':
        case 'Q':
          done = true;
          break;
        case ' ': 
          {
            static int  index = 0;
            static char str[255];
            sprintf (str,"live%02d.bmp", index++);
            image->writeToBMP (str);
          }
          break;
        }
      }
    }
  }


  // cleanup
  delete image;
  delete window;
  delete dispwin;
  delete camera;
  printf("done.\n");


  return 0;
}

