/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////



#include "tlDigitizer.h"
#include "tlDisplay.h"
#include "tlVision.h"



int
main (int  argc, 
      char **argv)
{
  int           done        = 0;
  int           res;
  char          msg[4];
  TL_DIGITIZER  *digitizer  = new TL_DIGITIZER ();
  tlDisplay     *window     = new tlDisplay ();
  tlImage       *image      = new tlImage ();
  tlImage       *grayImage  = new tlImage ();
  tlObject      *object     = new tlObject ();
  tlRect        *rect       = new tlRect ();


  // initialization
  tl_init_lib (argc, argv);
  digitizer->open ();
  digitizer->setFrameRate (60);
  window->show ();
  
  // random initial object definition
  digitizer->grab (image);
  image->toGray (grayImage);
  rect->set (60,30,50,60);
  object->define (grayImage, rect);
  
  // tracking loop
  while (!done) {

    // image acquisition
    digitizer->grab (image);
    image->toGray (rect, grayImage);
    
    // processing
    object->toRect (rect);
    rect->expand (10);
    res = grayImage->findBest (object, rect);
    sprintf (msg, "%02d%%", res);

    // display
    image->drawRect (object, TL_COLOR_GREEN);
    image->drawText (msg, object->x+object->width+2, object->y, TL_COLOR_WHITE, true);
    window->display (image);

    // event management
    if (tlDisplay::queryEvent()) {
      switch (tlDisplay::event()) {
      case TL_EVENT_EXIT:
      case TL_EVENT_KEYUP:
        done = 1;
        break;
      case TL_EVENT_MOUSERECT:
        tlDisplay::getMouseRect (rect);
        image->toGray (rect, grayImage);
        object->define (grayImage, rect);
      }
    }
  }


  return 0;
}
