// ==============================================================================================
// VRmUsbCam C API Win32-related Functions v2.6.0.0
// Copyright VRmagic 2004-2006
// ==============================================================================================

#ifndef VRMUSBCAM2WIN32_H
#define VRMUSBCAM2WIN32_H

#include <windows.h>

// ------------------------------------------------------------------------
// Device Property Page Support Functions
// ------------------------------------------------------------------------

/// This struct represents a device property page (device GUI), use VRmUsbCamCreateDevicePropertyPage() to create it
/// and destroy it afterwards with VRmUsbCamDestroyDevicePropertyPage(). NOTE: you may move,resize, hide/show the
/// page with Windows API calls using the m_handle Window Handle
typedef struct _VRmDevicePropertyPage {
	/// window handle of the page
	HWND m_handle;
	/// the size this page looks best
	VRmSizeI m_size_hint;
	/// API private informations
	void* mp_private;
} VRmDevicePropertyPage;

/// create a property page for the device.
/// if f_hwnd_parent is null, a new independent window is created, 
/// if f_hwnd_parent is not null, a child window is created within the given parent.
/// fp_rectangle may optionally point to a rectangle containing the initial window position.
/// NOTE: in case a new independent window is created, it is not shown automatically, so you need to show the window
/// by ShowWindow((*fpp_page)->m_handle,SW_SHOW)
VRM_API VRmRetVal VRmUsbCamCreateDevicePropertyPage(VRmUsbCamDevice f_device, HWND f_hwnd_parent, VRmRectI* fp_rectangle, VRmDevicePropertyPage** fpp_page);

/// destroy the given property page
VRM_API VRmRetVal VRmUsbCamDestroyDevicePropertyPage(VRmDevicePropertyPage** fpp_page);

#endif // VRMUSBCAM2WIN32_H
