/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLBLOBS_H__
#define __TLBLOBS_H__


#include "tlVision.h"



#ifndef NO_DOC
// blob list elements
typedef struct lblob{
  int    index;
  tlMask *mask;
  lblob  *next;
  lblob  *prev;
} lblob;
#endif

#define TL_MAX_LABELS  255



// class declaration
class tlBlobs
{
private:
  int    count;
	int    tcount;
	lblob  *current;
  lblob  *start;
  lblob  *end;
  int    connex;
  int    *labels;

  // internal chained list element creation
  struct lblob *newBlob (tlMask *mask);

  // internal management
  int remove  (lblob *blob);
  int swap    (lblob *lblob1, lblob* lblob2);

  // internal processing methods
  int processImage (tlImage *image, tlRect *rect, int minArea);
  int findBlobs    (tlImage *image, tlRect *rect);
  int extractBlobs (tlImage *image, tlRect *rect);

public:

  // constructors
  tlBlobs  ();
  tlBlobs  (tlImage *image, int minArea = 0);
  tlBlobs  (tlImage *image, tlRect *rect, int minArea = 0);
  virtual ~tlBlobs ();

  // data management
  int    getCount     () { return count;  }
  tlMask *getBlob     (int index);
  int    getBlob      (int index, tlMask *mask);
  tlMask *getBlobCopy (int index);

  // list management
  void setConnex (int connex) { this->connex = connex; };
  int  reset ();
  int  clear ();

  // blobs construction
  int extract (tlImage *image, int minArea = 0);
  int extract (tlImage *image, tlRect *rect, int minArea = 0);

  // blob area filters
  int filterArea    (int minArea, int maxArea);
  int filterMinArea (int minArea);
  int filterMaxArea (int maxArea);
  
  // aspect ratio filter
  int filterAspect    (float minRatio, float maxRatio);
  int filterMinAspect (float minRatio);
  int filterMaxAspect (float maxRatio);
  
  // surface area filter
  int filterSurface    (float minRatio, float maxRatio);
  int filterMinSurface (float minRatio);
  int filterMaxSurface (float maxRatio);
  
  // distance between blobs filter
  int filterDistance   (float minDist, float maxDist);
  int filterMinDist    (float minDist);
  int filterMaxDist    (float maxDist);
  
  // sort blobs by area
  int sortByArea ();

  // blob management
  int add     (tlMask *mask);
  int insert  (int index, tlMask *mask);
  int remove  (int index);
  int swap    (int index1, int index2);
};


#endif
