/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLCAMERASVSBASE_H__
#define __TLCAMERASVSBASE_H__


#define TL_CAMERA_SVS_FILENAME  "svs.ini"


#include "tlVision.h"
#include "tlCameraStereo.h"
#include "tlImage.h"

#include <string.h>		/* svsclass.h references strcpy */
#include "svsclass.h"


// debug verbosity
extern int tlVerbose;


// interface
class tlCameraSVSBase : public tlCameraStereo
{
protected:
  svsAcquireImages *sourceObject;
  svsStereoImage   *imageObject;
  svsMultiProcess  *processObject;
  svsSP            *sp;

  bool             color;
  bool             stoc;

public:
  tlCameraSVSBase  ();
  virtual ~tlCameraSVSBase ();

  int  setOffX      (int offX);
  int  setOffY      (int offY);
  int  setThreshold (int threshold);
  int  setNdisp     (int ndisp);
  int  getOffX      () { return imageObject->dp.offx; };
  int  getOffY      () { return imageObject->dp.offy; };
  int  getThreshold () { return imageObject->dp.thresh; };
  int  getNdisp     () { return imageObject->dp.ndisp; };
  bool isColor      () { return color; };
  bool isSTOC       () { return stoc; };

  virtual int readConfig   (char *filename);

  virtual int grab         (tlImage *image) = 0;
  virtual int grab         (tlImage *image, tlRect *rect) = 0;
  virtual int grabQueued   (tlImage *image) = 0;
  virtual int grabQueued   (tlImage *image, tlRect *rect) = 0;

  virtual int grab         (tlImage *image, tlImage *disparity) = 0;
  virtual int grab         (tlImage *image, tlImage *disparity, tlRect *rect) = 0;
  virtual int grabQueued   (tlImage *image, tlImage *disparity) = 0;
  virtual int grabQueued   (tlImage *image, tlImage *disparity, tlRect *rect) = 0;

  virtual int getRight     (tlImage *image);
  virtual int getRight     (tlImage *image, tlRect *rect);

  virtual int getDisparity (tlImage *disparity);
  virtual int getDisparity (tlImage *disparity, tlRect *rect);

  virtual int getDisparity (tlImage *rightImage, tlImage *leftImage, tlImage *disparity);
  virtual int getDisparity (tlImage *leftImage, tlImage *rightImage, tlImage *disparity, tlRect *rect);

  virtual int toWorld      (tlLocation *location);
  virtual int toWorld      (tlLocation *location, tlLocation *dest);
  virtual int toImage      (tlLocation *location);
  virtual int toImage      (tlLocation *location, tlLocation *dest);
};



#endif
