/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLCAMERASVSHEAD_H__
#define __TLCAMERASVSHEAD_H__


#include "tlVision.h"
#include "tlCameraSVSBase.h"
#include "tlImage.h"

#include "svsclass.h"


// debug verbosity
extern int tlVerbose;


// interface
class tlCameraSVSHead : public tlCameraSVSBase
{
private:
  svsVideoImages  *videoObject;

public:
  tlCameraSVSHead (int fps = 0);
  tlCameraSVSHead (char *filename, int fps = 0);
  virtual ~tlCameraSVSHead ();

  int  getGain           () { return videoObject->gain; };
  int  getExposure       () { return videoObject->exposure; };
  int  getContrast       () { return videoObject->contrast; };
  int  getBrightness     () { return videoObject->brightness; };
  int  getSaturation     () { return videoObject->saturation; };
  int  getRedGain        () { return videoObject->red; };
  int  getBlueGain       () { return videoObject->blue; };
  int  getSharpness      () { return videoObject->sharpness; };
  bool isAutoGain        () { return videoObject->use_autogain; };
  bool isAutoWhite       () { return videoObject->use_autowhite; };
  bool isAutoBrightness  () { return videoObject->use_autobrightness; };
  
  int  setGain           (int gain);
  int  setExposure       (int exposure);
  int  setContrast       (int contrast);
  int  setBrightness     (int brightness);
  int  setSaturation     (int saturation);
  int  setRedGain        (int red);
  int  setBlueGain       (int blue);
  int  setSharpness      (int sharpness);
  int  setAutoGain       (bool val);
  int  setAutoWhite      (bool val);
  int  setAutoBrightness (bool val);
  int  enableColor       (bool val);
  bool enableSTOC        (bool val);

  int  open  (int fps);
  int  open  (char *filename, int fps);
  int  close ();

  virtual int readConfig (char *filename);

  virtual int grab       (tlImage *image);
  virtual int grab       (tlImage *image, tlRect *rect);
  virtual int grabQueued (tlImage *image);
  virtual int grabQueued (tlImage *image, tlRect *rect);

  virtual int grab       (tlImage *image, tlImage *disparity);
  virtual int grab       (tlImage *image, tlImage *disparity, tlRect *rect);
  virtual int grabQueued (tlImage *image, tlImage *disparity);
  virtual int grabQueued (tlImage *image, tlImage *disparity, tlRect *rect);
};

#endif
