/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLCAMERATSAI_H__
#define __TLCAMERATSAI_H__


#include "tlVision.h"
#include "tlCamera.h"


#define TL_CAMERA_TSAI_FILENAME     "tsai.ini"
#define TL_CAMERA_TSAI_MAX_POINTS   500

// calibration data structure
typedef struct {
  double  Ncx;		    // [sel]     Number of sensor elements in camera's x direction
  double  Nfx;		    // [pix]     Number of pixels in frame grabber's x direction   
  double  dx;		      // [mm/sel]  X dimension of camera's sensor element (in mm)    
  double  dy;		      // [mm/sel]  Y dimension of camera's sensor element (in mm)    
  double  dpx;		    // [mm/pix]  Effective X dimension of pixel in frame grabber   
  double  dpy;		    // [mm/pix]  Effective Y dimension of pixel in frame grabber   
  double  Cx;		      // [pix]     Z axis intercept of camera coordinate system      
  double  Cy;		      // [pix]     Z axis intercept of camera coordinate system      
  double  sx;		      // []        Scale factor to compensate for any error in dpx   
  int     coplanar;   // []        1 if all input points are coplanar, 0 otherwise
  int     numPoints;
  double  xw[TL_CAMERA_TSAI_MAX_POINTS];    // world X coord
  double  yw[TL_CAMERA_TSAI_MAX_POINTS];    // world Y coord
  double  zw[TL_CAMERA_TSAI_MAX_POINTS];    // world Z coord
  double  Xf[TL_CAMERA_TSAI_MAX_POINTS];    // image X coord
  double  Yf[TL_CAMERA_TSAI_MAX_POINTS];    // image Y coord
} tlCameraTSAICalibData;



// debug verbosity
extern int tlVerbose;


// interface
class tlCameraTSAI : public tlCamera
{
private:
  tlSource *source;

public:
  tlCameraTSAI            ();
  tlCameraTSAI            (tlSource *source);
  tlCameraTSAI            (char *filename);
  tlCameraTSAI            (char *filename, tlSource *source);
  virtual ~tlCameraTSAI           ();

  int         readConfig  (char *filename);
  int         writeConfig (char *filename);
  int         calibrate   (tlCameraTSAICalibData *data);

  int         setSource   (tlSource *source);
  tlSource    *getSource  () { return source; };

  int         toWorld     (tlPoint *point);
  int         toImage     (tlPoint *point);

  virtual int warp        (tlPoint *point);
  virtual int warp        (tlPoint *point, tlPoint *dest);
  virtual int dewarp      (tlPoint *point);
  virtual int dewarp      (tlPoint *point, tlPoint *dest);
  
  virtual int warp        (tlImage *image);
  virtual int warp        (tlImage *image, tlImage *dest);
  virtual int dewarp      (tlImage *image);
  virtual int dewarp      (tlImage *image, tlImage *dest);

  virtual int warp        (tlImage *image, tlRect *rect);
  virtual int warp        (tlImage *image, tlRect *rect, tlImage *dest);
  virtual int dewarp      (tlImage *image, tlRect *rect);
  virtual int dewarp      (tlImage *image, tlRect *rect, tlImage *dest);

  virtual int grab        (tlImage *image);
  virtual int grab        (tlImage *image, tlRect *rect);
  virtual int grabQueued  (tlImage *image);
  virtual int grabQueued  (tlImage *image, tlRect *rect);

};

#endif

