/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLCHAIN_H__
#define __TLCHAIN_H__


#include "tlVision.h"


// corner extraction defaults
#define TL_CHAIN_CORNER_DMIN           7
#define TL_CHAIN_CORNER_DMAX           9
#define TL_CHAIN_CORNER_ALPHAMAX     150


#ifndef NO_DOC
// chained list elements
typedef struct lpoint{
  int     index;
  double  curvature;
  tlPoint *point;
  lpoint  *next;
  lpoint  *prev;
} lpoint;

// internal data struct for chain extraction
typedef struct {
  tlPixel *data;
  int width;
  int height;
  int x;
  int y;
  int x1; 
  int x2; 
  int y1; 
  int y2; 
  int connex;
} _tl_extraction_data;

#endif


// class declaration
class tlChain
{
private:
  int    length;
  int    closed;
  int    connex;
  lpoint *current;
  lpoint *start;
  lpoint *end;

  // internal chained list element creation
  struct lpoint *newPoint (tlPoint *point);
  struct lpoint *newPoint (int x, int y);

  // internal chain extraction recursion
  int extractPoint (_tl_extraction_data *exdat);

public:

  // constructors
  tlChain  ();
  tlChain  (tlPoint *start);
  tlChain  (int startx, int starty);
  virtual ~tlChain ();

  // data management
  int getLength  () { return length;  }
  int isClosed   () { return closed;  }
  int getConnex  () { return connex;  }
  
  tlPoint *getStart   () { return start->point; }
  tlPoint *getEnd     () { return end->point;   }
  tlPoint *getPoint   (int index);

  // chain management
  int reset ();
  int clear ();
  int filter (int radius);
  int close ();

  // point management
  int isIn    (tlPoint *point);
  int isIn    (int x, int y);
  int add     (tlPoint *point);
  int add     (int x, int y);
  int insert  (int index, tlPoint *point);
  int insert  (int index, int x, int y);
  int remove  (int index);
  int remove  (tlPoint *point);
  int remove  (int x, int y);

  // chain construction
  int extract      (tlImage *image, tlPoint *point, int connex = TL_CONNEX_8);
  int extract      (tlImage *image, int x, int y, int connex = TL_CONNEX_8);
  int extract      (tlImage *image, tlPoint *point, tlRect *rect, int connex = TL_CONNEX_8);
  int extract      (tlImage *image, int x, int y, tlRect *rect, int connex = TL_CONNEX_8);
  int blindExtract (tlImage *image, tlPoint *point, int connex = TL_CONNEX_8);
  int blindExtract (tlImage *image, int x, int y, int connex = TL_CONNEX_8);
  int blindExtract (tlImage *image, tlPoint *point, tlRect *rect, int connex = TL_CONNEX_8);
  int blindExtract (tlImage *image, int x, int y, tlRect *rect, int connex = TL_CONNEX_8);
  int extract      (tlMask *mask,   tlPoint *point, int connex = TL_CONNEX_8);
  int extract      (tlMask *mask,   int x, int y, int connex = TL_CONNEX_8);
  int extract      (tlMask *mask,   tlPoint *point, tlRect *rect, int connex = TL_CONNEX_8);
  int extract      (tlMask *mask,   int x, int y, tlRect *rect, int connex = TL_CONNEX_8);
  int blindExtract (tlMask *mask,   tlPoint *point, int connex = TL_CONNEX_8);
  int blindExtract (tlMask *mask,   int x, int y, int connex = TL_CONNEX_8);
  int blindExtract (tlMask *mask,   tlPoint *point, tlRect *rect, int connex = TL_CONNEX_8);
  int blindExtract (tlMask *mask,   int x, int y, tlRect *rect, int connex = TL_CONNEX_8);

  // corner detection
  int corners (tlChain *corners);
  int corners (int res, int alphaMax, tlChain *corners);
  int corners (int dmin, int dmax, int alphaMax, tlChain *corners);
};


#endif
