/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLCOLOR_H__
#define __TLCOLOR_H__


#include "tlVision.h"


// convenience defs for draw functions (r,g,b)
#define TL_COLOR_RED	      255,   0,   0   
#define TL_COLOR_GREEN        0, 255,   0     
#define TL_COLOR_BLUE	        0,   0, 255
#define TL_COLOR_WHITE      255, 255, 255
#define TL_COLOR_BLACK        0,   0,   0
#define TL_COLOR_ORANGE     255, 165,   0
#define TL_COLOR_GOLD       255, 215,   0
#define TL_COLOR_PINK	      255, 192, 203
#define TL_COLOR_YELLOW     255, 255,   0



class tlColor
{
public:

  // color format
  tl_format format;

  // raw (unspecified) format
  tlPixel raw[TL_MAX_CHANNEL];

  // GRAY format
  tlPixel intensity;

  // RGB format
  tlPixel r;
  tlPixel g;
  tlPixel b;

  // YUV format
  tlPixel y;
  tlPixel u;
  tlPixel v;

  // NRG format
  tlPixel nr;
  tlPixel ng;

  // HSI format
  tlPixel h1;
  tlPixel h2;
  tlPixel s;
  tlPixel i;

  
  // constructors
  tlColor  ();
  tlColor  (tlColor *src);
  tlColor  (tlPixel r,  tlPixel g, tlPixel b);
  tlColor  (tlPixel h1, tlPixel h2, tlPixel s, tlPixel i);
  tlColor  (tlPixel nr, tlPixel ng);
  tlColor  (tlPixel intensity);
  virtual ~tlColor ();
  
  // management
  int copy   (tlColor *color);
  int copyTo (tlColor *color);

  // channel manageent
  int setChannel   (int channel, tlPixel val);
  int swapChannels (int channel1, int channel2);

  // set color from image
  int set (tlPixelArray *pixelArray, tlPoint *point);
  int set (tlPixelArray *pixelArray, int x, int y);

  // set color from value
  int setRgb  (tlPixel r , tlPixel g , tlPixel b);
  int setYuv  (tlPixel y , tlPixel u , tlPixel v);
  int setNrg  (tlPixel nr, tlPixel ng);
  int setHsi  (tlPixel h1, tlPixel h2, tlPixel s, tlPixel i);
  int setGray (tlPixel intensity);

  // set color from image
  int setMin  (tlPixelArray *pixelArray, tlRect *rect);
  int setMax  (tlPixelArray *pixelArray, tlRect *rect);
  int setVar  (tlPixelArray *pixelArray, tlRect *rect);

  // change format (color format conversion)
  int toRgb   ();
  int toYuv   ();
  int toNrg   ();
  int toHsi   ();
  int toGray  ();
  int toBin   (int threshold);

  // add and subtract colors
  int add     (tlColor *color);
  int add     (tlColor *color, tlColor *dest);
  int sub     (tlColor *color);
  int sub     (tlColor *color, tlColor *dest);
};


#endif

