/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLCONST_H__
#define __TLCONST_H__


#include "tlVision.h"


template<typename T>
class tlConst
{
private:
  T     val;
  T     pred;
  int   tol;          // tolerance [%]
	float s, s_;        // state and state estimate
	float e, e_;        // estimate error covariance (before/after)
	float p, m;         // process and measurement covariances

public:

  // constructors
  tlConst  ();
  tlConst  (T val, float pCv, int tol=-1);
  tlConst  (T val, float eCv, float pCv, float mCv, int tol=-1);
  virtual ~tlConst ();

  // initialization
  int init (T val, float pCv, int tol=-1);
  int init (T val, float eCv, float pCv, float mCv, int tol=-1);

  // management
  int setVal   (T val);
  int setParam (float pCv, int tol=-1);
  int setParam (float eCv, float pCv, float mCv, int tol=-1);
  int setTol   (int tol);

  // computation
  int   update      (T val);

  // data access
  int   getTol      () { return tol;  }
  T     getVal      () { return val;  }
  T     getNext     () { return pred; }
  float getState    () { return s;    }
  float getEstimate () { return s_;   }
};


#endif

