/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLDIGITIZER_H__
#define __TLDIGITIZER_H__


#include "tlVision.h"
#include "tlSource.h"


// auto settings
#define TL_SOURCE_AUTO        0x1

// input format (can be cumulated)
#define TL_SOURCE_COMPOSITE   0x2
#define TL_SOURCE_SVIDEO      0x4
#define TL_SOURCE_TVTUNER     0x8
#define TL_SOURCE_DIGITAL     0x16
#define TL_SOURCE_NTSC        0x32
#define TL_SOURCE_NTSCJ       0x64
#define TL_SOURCE_PAL         0x128
#define TL_SOURCE_PALM        0x256
#define TL_SOURCE_PALN        0x512
#define TL_SOURCE_SECAM       0x1024


// debug verbosity
extern int tlVerbose;


// interface
class tlDigitizer : public tlSource
{
protected:
  bool           isOpen;
  unsigned short input;
  int            channel;
  int            fps;

public:
  tlDigitizer ();
  tlDigitizer (int width, int height);
  virtual ~tlDigitizer ();

  virtual int open  (int index = 0)                        = 0;
  virtual int open  (int width, int height, int index = 0);
  virtual int close ()                                     = 0;

  unsigned short
      getInput      () { return input;  };
  int getChannel    () { return channel; };
  int getFrameRate  () { return fps;     };

  virtual int setChannel    (int channel, unsigned short input);
  virtual int setInput      (unsigned short input);
  virtual int setFrameRate  (int fps);
  virtual int setSize       (int width, int height);

  virtual int grab        (tlImage *image)                 = 0;
  virtual int grab        (tlImage *image, tlRect *rect)   = 0;
  virtual int grabQueued  (tlImage *image)                 = 0;
  virtual int grabQueued  (tlImage *image, tlRect *rect)   = 0;
};


//
// default digitizer shortcuts
//

#ifndef _TLIB_BUILD_

#ifdef WIN32
#include "tlDigitizerDX.h"
#define TL_DIGITIZER tlDigitizerDX
#endif

#ifdef LINUX
#include "tlDigitizerV4L2.h"
#define TL_DIGITIZER tlDigitizerV4L2
#endif

#endif

#endif
