/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://imtsg14.epfl.ch/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLDIGITIZERV4L2_H__
#define __TLDIGITIZERV4L2_H__

/* LINUX is defined in the makefile */
#if    !LINUX
#error THIS IS A LINUX VERSION 
#endif

#define TLDIGITIZER_V4L2_BASE_NAME "/dev/video"

#include <sys/types.h>
#include <asm/types.h>          /* for videodev2.h */
#include <linux/videodev2.h>

#include "tlVision.h"
#include "tlDigitizer.h"


// interface
class tlDigitizerV4L2 : public tlDigitizer
{
private:
  struct buffer {
    void   *start;
    size_t length;
  };
  int                        ifd;
  struct buffer              *buffers;
  unsigned int               n_buffers;
  struct v4l2_buffer	     mbuf;
  struct v4l2_requestbuffers vbuf;
  unsigned char		     *rgbImage;
  struct v4l2_capability     vcap;
  int                        grabQueuedInit;
  int                        channel;
  int                        format;
  
  void init_mmap       (void);
  void uninit_mmap     (void);
  void start_capturing (void);
  void stop_capturing  (void);
  
public:
  tlDigitizerV4L2        ();
  tlDigitizerV4L2        (int width, int height);
  tlDigitizerV4L2        (int width, int height, int channel, unsigned short input = TL_SOURCE_AUTO);
  ~tlDigitizerV4L2       ();

  virtual int open       (int index = 0);
  virtual int open       (int width, int height, int index = 0);
          int open       (int width, int height, int index, int channel, unsigned short input = TL_SOURCE_AUTO);
  virtual int close      ();

  virtual int setChannel (int channel, unsigned short input);
  virtual int setInput   (unsigned short input);
  virtual int setSize    (int width, int height);

  virtual int grab       (tlImage *image);
  virtual int grab       (tlImage *image, tlRect *rect);
  virtual int grabQueued (tlImage *image);
  virtual int grabQueued (tlImage *image, tlRect *rect);
};


#endif
