/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLDISPLAY_H__
#define __TLDISPLAY_H__



#include "tlVision.h"
#include "tlSize.h"
#include "tlLocation.h"
#include "tlRect.h"




// constants
#define TL_MAX_DISPLAY           10
#define TL_DISPLAY_NAME         "TLIB window"


// event list
#define TL_NO_EVENT               0
#define TL_EVENT                  1
#define TL_EVENT_EXIT             2
#define TL_EVENT_MOUSE            3
#define TL_EVENT_MOUSEDOWN        4
#define TL_EVENT_MOUSEUP          5
#define TL_EVENT_LMOUSE           6
#define TL_EVENT_LMOUSEDOWN       7
#define TL_EVENT_LMOUSEUP         8
#define TL_EVENT_RMOUSE           9
#define TL_EVENT_RMOUSEDOWN      10
#define TL_EVENT_RMOUSEUP        11
#define TL_EVENT_MMOUSE          12
#define TL_EVENT_MMOUSEDOWN      13
#define TL_EVENT_MMOUSEUP        14
#define TL_EVENT_MOUSEDBLCLK     15
#define TL_EVENT_MOUSERECT       16
#define TL_EVENT_KEY             17
#define TL_EVENT_KEYDOWN         18
#define TL_EVENT_KEYUP           19



// class declaration
class tlDisplay
{

private:
  // window
  int  x;
  int  y;
  int  width;
  int  height;
  int  ID;
  bool visible;
  char *name;

  int  init   (char *filename, int x, int y);
  int  update ();

public:
  tlDisplay  ();
  tlDisplay  (int x, int y);
  tlDisplay  (char *name);
  tlDisplay  (char *name, int x, int y);
  virtual ~tlDisplay ();

  // window routines
  bool getVisible () { return visible; };
  int  getID      () { return ID;  };
  int  getWidth   () { return width;  };
  int  getHeight  () { return height; };
  int  getPosX    () { return x; }
  int  getPosY    () { return y; }

  int  getSize    (tlSize *size)          { return size->setSize (width, height); }
  int  getPos     (tlLocation *location)  { return location->setLocation (x, y); }
  int  getRect    (tlRect *rect)          { return rect->set (x, y, width, height); }

  int  setPos     (int x, int y);
  int  resize     (int width, int height);
  int  maximize   ();
  int  restore    ();
  int  minimize   ();
  int  close      ();
  int  show       ();
  int  hide       ();
  int  display    (tlPixelArray *pixelArray);
  int  display    (tlHist       *hist);
  int  display    (tlHist2D     *hist);

  // event management routines
  static bool       init_global;
  static bool       isInitialized  () { return init_global; };
  static void       initialize     ();
  static int        queryEvent     ();
  static int        event          ();
  static int        getLastEvent   ();
  static tlDisplay* getLastDisplay ();
  static tlPoint*   getMouse       ();
  static int        getMouse       (tlPoint *point);
  static tlPoint*   getMouseDown   ();
  static int        getMouseDown   (tlPoint *point);
  static tlPoint*   getMouseUp     ();
  static int        getMouseUp     (tlPoint *point);
  static tlPoint*   getMouseDouble ();
  static int        getMouseDouble (tlPoint *point);
  static tlRect*    getMouseRect   ();
  static int        getMouseRect   (tlRect  *rect);
  static char       getKeyDown     (); 
  static char       getKeyUp       ();
  static int        setMousePos    (int x, int y);
  static char       waitKey        ();
  static void       waitMouse      ();
};

#endif

