/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLERROR_H__
#define __TLERROR_H__

#include "tlVision.h"


// error codes
enum tl_errors { 
  TL_NO_ERROR, 
  TL_ERROR, 
  TL_ERROR_INVALID_FORMAT,
  TL_ERROR_GEOMETRY, 
  TL_ERROR_NULL_POINTER,
  TL_ERROR_INDEX_OUT_OF_BOUNDS,
  TL_ERROR_MAX_REACHED,
  TL_ERROR_ALLOCATION_FAILED,
  TL_ERROR_NOT_IMPLEMENTED, 
  TL_ERROR_FILE_IO, 
  TL_ERROR_FILE_NOT_FOUND_OR_CORRUPT, 
  TL_ERROR_NOT_REQUIRED,
  TL_ERROR_NOT_FOUND, 
  TL_ERROR_NOT_AVAILABLE, 
  TL_ERROR_NOT_IN_IMAGE,
  TL_ERROR_STACK_OVERFLOW, 
  TL_ERROR_TIMEOUT,
  TL_ERROR_THREAD,
  TL_ERROR_TIMER,
  TL_ERROR_RESOURCE_NOT_AVAILABLE,
  TL_ERROR_CONFIG_FAILED,
  TL_ERROR_GRAB_FAILED,
  TL_ERROR_HIGHRES_INCOMPATIBLE,
  TL_ERROR_NA
};

#define tl_errno (*tl_errno_location())
#define TL_ERROR_MSG(msg)       { if (tlVerbose) { cout << msg << endl;} }
#define TL_ERRNO_MSG(msg,errno) { (tl_errno = errno); if (tlVerbose > 0) {tlError (msg); }}

// tl_errno thread-safe global
int *tl_errno_location ();
void tl_errno_destructor (void *);
void tl_errno_once ();

// error processing
void tlError ();
void tlError (const char *str);
void tlError (int errno);
void tlError (int errno, const char *str);
char *tlErrorGetString (int errno);
char *tlErrorGetString (int errno);

#endif
