/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLHISTOGRAM_H__
#define __TLHISTOGRAM_H__


#include "tlVision.h"


class tlHist
{
private:

  int         length;			// histogram length (i.e. number of elements)
  int         offset;			// data offset (i.e. first index)
  int         min;				// index of min value
  int         max;				// index of max value
  int         mean;       // mean (integer) value
  int         weight;     // histogram weight (sum of all elements)
  float       stdDev;     // standard deviation
  tlHistData  minVal;			// min value
  tlHistData  maxVal;			// max value

  int create      (int length, int offset);
  int deleteData  ();

public:

  int         channel;	  // for information only
  tlHistData  *data;			// pointer to histogram data
  tlHistData  *rank;      // pointer to histogram ranking information

    
  // constructors
  tlHist  ();
  tlHist  (int length, int offset = 0);
  virtual ~tlHist ();

  // initialization
  int   init       (int length, int offset = 0);
  int   clear      ();
  int   update     ();

  // set data
  int   setChannel (int channel);

  // read data
  int   getVal      (int index);
  int   getRank     (int index);
  int   getMean     () { return mean;   };
  float getStdDev   () { return stdDev; };
  int   getMaxVal   () { return maxVal; };
  int   getMinVal   () { return minVal; };
  int   getMaxIndex () { return max;    };
  int   getMinIndex () { return min;    };
  int   getOffset   () { return offset; };
  int   getLength   () { return length; };

  // set data
  int   setVal (int index, int val);
  int   incVal (int index);
  int   decVal (int index);

  // normalize [% peak]
  int   normalize (int max = 100);

  // normalize [% sum]
  int   integrate (int sum = 100);

  // compare
  float compare (tlHist *hist);

  // build histogram
  int   build (tlPixel *data, int len);
  int   build (tlPixel *data, int len, int length, int offset = 0);
  int   build (tlPixel *src, int width, int height, int pixelWidth, tlPixel *mask, int channel, tlRect *rect);

  // build histogram for high-resolution images
  int   build (tlHRPixel *data, int len);
  int   build (tlHRPixel *data, int len, int length, int offset = 0);
  int   build (tlHRPixel *src, int width, int height, int pixelWidth, tlPixel *mask, int channel, tlRect *rect);
};


#endif
