/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLHISTOGRAM2D_H__
#define __TLHISTOGRAM2D_H__


#include "tlVision.h"



class tlHist2D
{
private:

  int         xLength;    // histogram length (i.e. number of elements) on X
  int         yLength;    // histogram length (i.e. number of elements) on Y
  int         xOffset;    // data offset (i.e. first index) on X
  int         yOffset;    // data offset (i.e. first index) on Y
  int         xMin;				// X index of min value
  int         xMax;				// X index of max value
  int         yMin;				// Y index of min value
  int         yMax;				// Y index of max value
  tlHistData  minVal;			// min value
  tlHistData  maxVal;			// max value

  int create      (int xlength, int ylength, int xoffset, int yoffset);
  int deleteData  ();

public:

  int         xChannel;	  // for information only
  int         yChannel;	  // for information only
  tlHistData  **data;			// pointer to histogram data
  tlHistData  *xRank;     // pointer to histogram ranking information
  tlHistData  *yRank;     // pointer to histogram ranking information

    
  // constructors
  tlHist2D  ();
  tlHist2D  (int length);
  tlHist2D  (int xlength, int ylength, int xoffset = 0, int yoffset = 0);
  virtual ~tlHist2D ();

  // file read/write
  int save (char *filename);
  int load (char *filename);

  // initialization
  int   init       (int length);
  int   init       (int xlength, int ylength, int xoffset = 0, int yoffset = 0);
  int   clear      ();
  int   update     ();
  int   updateRank ();

  // set data
  int   setXChannel (int channel);
  int   setYChannel (int channel);

  // read data
  int   getVal        (int x, int y);
  int   getXRank      (int index);
  int   getYRank      (int index);
  int   getMaxVal     () { return maxVal; };
  int   getMinVal     () { return minVal; };
  int   getXOffset    () { return xOffset; };
  int   getXLength    () { return xLength; };
  int   getXMaxIndex  () { return xMax;    };
  int   getXMinIndex  () { return xMin;    };
  int   getYOffset    () { return yOffset; };
  int   getYLength    () { return yLength; };
  int   getYMaxIndex  () { return yMax;    };
  int   getYMinIndex  () { return yMin;    };

  // set data
  int   setVal (int x, int y, tlHistData val);
  int   incVal (int x, int y);
  int   decVal (int x, int y);

  // normalize [% peak]
  int   normalize (int max = 100);

  // normalize [% sum]
  int   integrate (int sum = 100);

  // compare
  float compare (tlHist2D *hist);

  // build histogram
  int   build (tlPixel *xdata, tlPixel *ydata, int len);
  int   build (tlPixel *xdata, tlPixel *ydata, int len, int xlength, int ylength, int xoffset = 0, int yoffset = 0);
  int   build (tlPixel *src, int width, int height, int pixelWidth, tlPixel *mask, int xchannel, int ychannel, tlRect *rect);

  // build histogram from high-resolution images
  int   build (tlHRPixel *xdata, tlHRPixel *ydata, int len);
  int   build (tlHRPixel *xdata, tlHRPixel *ydata, int len, int xlength, int ylength, int xoffset = 0, int yoffset = 0);

  // project to 1D histograms
  int     getXHist (tlHist *hist);
  int     getYHist (tlHist *hist);
  tlHist *getXHist ();
  tlHist *getYHist ();

  // project to image
  int     toImage  (tlImage *image);
  tlImage *toImage ();
};

#endif
