/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLHOUGH_H__
#define __TLHOUGH_H__


#include <vector>
using std::vector;

#include "tlVision.h"



// class declaration
class tlHough
{
public:
  tlImage *srcImage;
  int     srcWidth;
  int     srcHeight;
  short   *accumulator;
  short   *sinTheta;
  short   *cosTheta;
  float   angularIncrement;
  int     hOffset;
  int     hWidth;
  int     hHeight;
  short   maxAcc;

  vector <tlLine*> lineList;

private:
  int clear();
  int add (tlLine *line);
  struct lline *newLine (tlLine *line);
  int makeAccumulator ();
  int clearAccumulator ();
  int extractLines (int lineCount);

public:
  // *structors
  tlHough (int sWidth, int sHeight, float angIncr = 0.0f);
  virtual ~tlHough();

  // define
  int   setParam            (int sWidth, int sHeight, float angIncr = 0.0f);
  int   setAngularIncrement (float angIncr);
  float getAngularIncrement () { return (float)(180.0f*angularIncrement/TL_PI); };

  // transforms
  int    extract     (tlImage *image, int lineCount = 1);
  
  // retrieve
  int    getLineCount ();
  tlLine *getLine     (int index);
  tlLine *getLinePtr  (int index);
  int    getLineCopy  (tlLine *line, int index);

  // image
  int getHoughImage (tlImage *image);
};


#endif
