/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLIMAGE_H__
#define __TLIMAGE_H__


#include "tlVision.h"
#include "tlPixelArray.h"


class tlImage : public tlPixelArray
{
public:

  // constructors
  tlImage  ();
  tlImage  (tl_format format, int width, int height);
  tlImage  (tlImage *image);
  virtual ~tlImage ();

  // disk storage
  int writeToFile  (char *filename);
  int writeToPPM   (char *filename);
  int writeToBMP   (char *filename);
  int writeToJPG   (char *filename, int quality = 80);
  int readFromFile (char *filename);
  int readFromPPM  (char *filename);
  int readFromBMP  (char *filename);
  int readFromJPG  (char *filename);

  // overlay drawing
  int drawPoint  (int x, int y, unsigned char r, unsigned char g, unsigned char b);
  int drawPoint  (int x, int y, unsigned char v);
  int drawPoint  (tlPoint *point, unsigned char r, unsigned char g, unsigned char b);
  int drawPoint  (tlPoint *point, unsigned char v);
  int drawLine   (int x1, int y1, int x2, int y2, unsigned char r, unsigned char g, unsigned char b);
  int drawLine   (int x1, int y1, int x2, int y2, unsigned char v);
  int drawLine   (tlPoint *p1, tlPoint *p2, unsigned char r, unsigned char g, unsigned char b);
  int drawLine   (tlPoint *p1, tlPoint *p2, unsigned char v);
  int drawLine   (tlLine *line, unsigned char r, unsigned char g, unsigned char b);
  int drawLine   (tlLine *line, unsigned char v);
  int drawRect   (tlRect *rect, unsigned char r, unsigned char g, unsigned char b);
  int drawRect   (tlRect *rect, unsigned char v);
  int drawCross  (int x, int y, int size, unsigned char r, unsigned char g, unsigned char b);
  int drawCross  (int x, int y, int size, unsigned char v);
  int drawCross  (tlPoint *point, int size, unsigned char r, unsigned char g, unsigned char b);
  int drawCross  (tlPoint *point, int size, unsigned char v);
  int drawCross  (tlRect *rect, unsigned char r, unsigned char g, unsigned char b);
  int drawCross  (tlRect *rect, unsigned char v);
  int drawCross  (tlChain *chain, int size, unsigned char r, unsigned char g, unsigned char b);
  int drawCross  (tlChain *chain, int size, unsigned char v);
  int drawMask   (tlMask *mask, unsigned char r, unsigned char g, unsigned char b);
  int drawMask   (tlMask *mask, unsigned char v);
  int drawMask   (tlMask *mask, int x, int y, unsigned char r, unsigned char g, unsigned char b);
  int drawMask   (tlMask *mask, int x, int y, unsigned char v);
  int drawChain  (tlChain *chain, unsigned char r, unsigned char g, unsigned char b);
  int drawChain  (tlChain *chain, unsigned char v);
  int drawObject (tlObject *object);
  int drawObject (tlObject *object, int x, int y);
  int drawImage  (tlImage *image, int x, int y);
  int drawImage  (tlImage *image, tlLocation *location);
  int fillRect   (tlRect *rect, unsigned char r, unsigned char g, unsigned char b);
  int fillRect   (tlRect *rect, unsigned char v);
  int fillObject (tlObject *object, unsigned char r, unsigned char g, unsigned char b);
  int fillObject (tlObject *object, unsigned char v);
  int fillObject (tlObject *object, int x, int y, unsigned char r, unsigned char g, unsigned char b);
  int fillObject (tlObject *object, int x, int y, unsigned char v);
  int drawText   (char *text, int  x, int  y, unsigned char r, unsigned char g, unsigned char b, bool blackbg = false);
  int drawText   (char *text, int  x, int  y, unsigned char v, bool blackbg = false);
  int drawText   (char *text, tlPoint *p, unsigned char r, unsigned char g, unsigned char b, bool blackbg = false);
  int drawText   (char *text, tlPoint *p, unsigned char v, bool blackbg = false);

  // image normalizing
  int normalize ();
  int normalize (tlRect *rect);
  int normalize (tlMask *mask);
  int normalize (tlMask *mask, tlRect *rect);
  int normalize (tlImage *dest);
  int normalize (tlRect *rect, tlImage *dest);
  int normalize (tlMask *mask, tlImage *dest);
  int normalize (tlMask *mask, tlRect *rect, tlImage *dest);

  // edge detection
  int edges     (int method=TL_EDGES_SOBEL);
  int edges     (tlRect *rect, int method=TL_EDGES_SOBEL);
  int edgesVert (int method=TL_EDGES_SOBEL);
  int edgesVert (tlRect *rect, int method=TL_EDGES_SOBEL);
  int edgesHor  (int method=TL_EDGES_SOBEL);
  int edgesHor  (tlRect *rect, int method=TL_EDGES_SOBEL);

  // edge detection to a new image
  int edges     (tlImage *dest, int method=TL_EDGES_MORPH);
  int edges     (tlRect *rect, tlImage *dest, int method=TL_EDGES_MORPH);
  int edgesVert (tlImage *dest, int method=TL_EDGES_SOBEL);
  int edgesVert (tlRect *rect, tlImage *dest, int method=TL_EDGES_SOBEL);
  int edgesHor  (tlImage *dest, int method=TL_EDGES_SOBEL);
  int edgesHor  (tlRect *rect, tlImage *dest, int method=TL_EDGES_SOBEL);

  // borders
  int borderBlack   (int b);
  int borderBlack   (int bx, int by);
  int borderMirror  (int b);
  int borderMirror  (int bx, int by);

  // borders to new image
  int crop          (tlRect *rect);
  int crop          (int x, int y, int width, int height);
	int crop          (tlRect *rect, tlImage *dest);
  int crop          (int x, int y, int width, int height, tlImage *dest);
  int borderBlack   (int b, tlImage *dest);
  int borderBlack   (int bx, int by, tlImage *dest);
  int borderMirror  (int b, tlImage *dest);
  int borderMirror  (int bx, int by, tlImage *dest);

  // mirror effect
  int mirror        ();
  int mirror        (tlImage *dest);

  // negative effect
  int invert        ();
  int invert        (tlImage *dest);

  // rotation effect
  int rotateRight ();
  int rotateLeft  ();
  int rotateRight (tlImage *dest);
  int rotateLeft  (tlImage *dest);
};


#endif

