/////////////////////////////////////////////////////////////////////////////////
//
//  This file is part of the TLIB computer vision library.
//  Copyright (C) 2003-2008 Sebastien Grange
//  Copyright (C) 2003-2007 VRAI Group, EPFL
//  All rights reserved.
// 
//  This library is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License("GPL") version 2
//  as published by the Free Software Foundation.
// 
//  <http://www.tuyphon.com/tlib>
//
/////////////////////////////////////////////////////////////////////////////////


#ifndef __TLMASK_H__
#define __TLMASK_H__


#include "tlVision.h"
#include "tlPixelArray.h"



class tlMask : public tlPixelArray
{
public:

  int       cx;
  int       cy;
  int       area;


  // constructors
  tlMask  ();
  tlMask  (int cx, int cy, int width, int height);
  tlMask  (tlMask *mask);
  tlMask  (tlImage *image);
	virtual ~tlMask ();

  // mask management
  int set         ();
  int set         (int width, int height);
  int set         (tlMask *mask);
  int set         (char *mask, int width, int height);
  int set         (char *mask, int width, int height, tlRect *rect);
  int set         (tlPixel *mask, int width, int height);
  int set         (tlPixel *mask, int width, int height, tlRect *rect);
  int set         (tlObject *object);
  int set         (tlImage *image);
  int set         (tlImage *image, tlRect *rect);
  int invert      ();
  int invert      (tlRect *rect);
	int setCenter   ();
  int setCenter   (int cx, int cy);
  int clear       ();
  int clear       (int width, int height);

  // mask operations
  int fill        ();
  int crop        (int val);

  // interaction with other masks
  int merge       (tlMask *mask);

	// adjust to a rectangular component
	int adjust      (tlRect *rect);

	// set to minimal size
	int minimize    ();

  // misc
  int copy        (tlMask *mask);
  int copyTo      (tlMask *mask);
};



#endif

